/*
  Copyright Red Hat, Inc. 2008

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; version 2.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to the
  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
  MA 02139, USA.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "metadata.h"
#include "res-ocf.h"

void
xml_print_desc(ra_desc_t *desc) 
{
	char *lang="C";

	if (desc->d_lang)
		lang = desc->d_lang;
	
	if (desc->d_longdesc) {
		printf("\t\t<longdesc lang=\"%s\">%s</longdesc>\n",
			lang, desc->d_longdesc);
	}
	if (desc->d_shortdesc) {
		printf("\t\t<shortdesc lang=\"%s\">%s</shortdesc>\n",
			lang, desc->d_longdesc);
	}
}	


void
xml_print_parameter(ra_param_t *param) 
{
	int x;

	printf("\t<parameter name=\"%s\" ", param->param_name);
	if (param->param_inherit)
		printf("inherit=\"%s\" ", param->param_inherit);
	if (param->param_flags & RA_FL_PRIMARY) {
		printf("primary=\"1\" ");
	}
	if (param->param_flags & RA_FL_UNIQUE) {
		printf("unique=\"1\" ");
	}
	if (param->param_flags & RA_FL_REQUIRED) {
		printf("required=\"1\" ");
	}
	printf(" >\n");

	for (x = 0; param->param_descs[x].d_lang; x++) {
		xml_print_desc(&param->param_descs[x]);
	}

	if (param->param_content_type) {
		printf("\t\t<content type=\"%s\" ", param->param_content_type);
		if (param->param_default)
			printf("default=\"%s\" ", param->param_default);
		printf("/>\n");
	}

	printf("\t</parameter>\n");

}	


void
xml_print_action(ra_act_t *act)
{
	printf("\t\t<action name=\"%s\" ", act->a_name);
	if (act->a_timeout)
		printf("timeout=\"%s\" ", act->a_timeout);
	if (act->a_interval)
		printf("interval=\"%s\" ", act->a_interval);
	if (act->a_depth)
		printf("depth=\"%s\" ", act->a_depth);
	printf("/>\n");
}

void
xml_print_metadata(ra_info_t *agent) 
{
	int x;

	printf("<?xml version=\"1.0\"?>\n");
	printf("<resource-agent name=\"%s\" version=\"%s\" >\n",
		agent->ra_header->h_name,
		agent->ra_header->h_provider_version);
	printf("\t<version>%s.%s</version>\n",
		agent->ra_header->h_version_major,
		agent->ra_header->h_version_minor);

	for (x = 0; agent->ra_desc[x].d_lang; x++)
		xml_print_desc(&agent->ra_desc[x]);

	printf("\t<parameters>\n");

	for (x = 0; agent->ra_params[x].param_name; x++)
		xml_print_parameter(&agent->ra_params[x]);

	printf("\t</parameters>\n");

	printf("\t<actions>\n");

	for (x = 0; agent->ra_acts[x].a_name; x++)
		xml_print_action(&agent->ra_acts[x]);

	printf("\t</actions>\n");

	printf("</resource-agent>\n");
}


int
get_param(ra_info_t *ra, const char *name, char *buf, size_t buflen)
{
	char env_name[128];
	char *tmp;
	int len;

	snprintf(env_name, sizeof(env_name), "%s%s", OCF_RES_PREFIX, name);
	tmp = getenv(env_name);
	if (!tmp) {
		for (len = 0; ra->ra_params[len].param_name; len++) {
			if (strcasecmp(ra->ra_params[len].param_name, name))
				continue;
			tmp = ra->ra_params[len].param_default;
			if (!tmp)
				return 1;
		}
	}

	len = strlen(tmp);
	if (len) {
		if (len > (buflen+1)) {
			errno = ENAMETOOLONG;
			return -1;
		}
		snprintf(buf, buflen, "%s", tmp);
		return 0;
	}

	return 1;
}
